
/**
 * \mainpage ELL - Energy Landscape Library
 *
 * <center> ELL - 3.2.2 </center>
 *
 * This document provides reference information about the 
 * <A HREF="http://www.bioinf.uni-freiburg.de/SW/ELL/">Energy Landscape Library</A>.
 * 
 * \section SecGoals The goal
 * 
 * This C++ programming library provides a platform for generic algorithms to 
 * study kinetics and structure of energy landscapes with descrete states. This 
 * algorithms need an abstract representation of this states to be applied to
 * a multitude of models and the corresponding energy or fitness landscapes.
 * 
 * \section SecFeatures Features
 *
 * \subsection SubSecAlgos Generic algorithms
 *  
 * <DIV class="subsection">
 * 
 * A set of walks to traverse energy landscapes:
 * - random walks
 * - adaptive walks
 * - gradient (steepest descent) walks
 * - Monte Carlo walks with Metropolis criterion
 * - low energy path calculation for RNA landscapes using the heuristic by
 *   Morgan and Higgs
 *
 * A walk based barrier estimation in the landscape
 * - a direct walk based estimation using pairs of minima (e.g. Morgan-Higgs
 *   heuristic for RNA landscapes)
 * - an up- and down-walk based estimation starting in a known minimum of
 *   landscape and ending in another one. If this is unknown it can be added.
 * 
 * Exhaustive landscape flooding:
 * - flooding of single basins (standard via hash and a more memory efficient version)
 * - flooding of multiple basins in parallel (in preparation)
 *
 * Transition probability estimation:
 * - fast estimation of all transition probabilities to directly adjacent 
 *   macro-states of a given one based on a Metropolis Monte-Carlo simulation
 *
 * All algorithms are fully parameterisable and very flexible concerning output
 * and results. So they can be set up to:
 * - write all visited states to stream
 * - collect them in listings (also compressed possible)
 * - calculate the canonical partition function of the visited states (e.g. of
 *   a gradient basin of a minimum)
 * - perform any wanted operation by overwriting the corresponding reporter 
 *   classes
 *
 * </DIV>
 * 
 * \subsection SubSecStates States = Landscape Models
 * 
 * - Lattice protein representations with Pull-Move and Pivot-Move neighborhood for different 
 *   lattice models with arbitrary distance based energy functions (based on the 
 *   <A HREF="http://www.bioinf.uni-freiburg.de/sw/biu/"> BIU </A> library) 
 * - an RNA representation interfacing the <A HREF="http://www.tbi.univie.ac.at/~ivo/RNA/">
 *   Vienna RNA package </A> for energy calculation
 * - State for Ising spin glass models (ring of spins)
 * - State for the Number Partitioning Problem (ring of spins) 
 * - A generic State implementation based on a node-labeled graph. Using this class
 *   any arbitrary energy landscape can be encoded and directly used within the
 *   ELL. Each node represents a State and is labeled with its energy while the
 *   edges encode the neighborhood relation of the energy landscape.
 *
 * \subsection SubSecTopologies Landscape Topology Representations
 *
 * To allow for a coarse grained landscape topology representation of minima
 * and additional information as barriers, saddle points, etc. we provide a set
 * of representations via subclassing of LandscapeTopology:
 *
 * - set of minima
 * - barrier tree 
 * - saddle network 
 * - funnel partitioning (in preparation)
 * 
 * \subsection SubSecGeneral General features
 * 
 * - compressed State representations to minimize the memory usage for storing
 *   huge numbers of states (e.g. for energy landscape flooding)
 * - generic neighborhood generators
 * - the random number generation can be globally set or one of the already 
 *   available generators of the BIU library can be used
 * - tests for most of the given functionality to enable a high standard of the
 *   source code. Furthermore, these tests can serve as an example for the own 
 *   first steps using the ELL.
 *
 * 
 *
 * \section SecDependencies Dependencies
 * 
 * The ELL depends on the following libraries:
 * 
 * - the <A HREF="http://www.bioinf.uni-freiburg.de/SW/BIU/">Bioinformatic Utility</A>
 *   library (BIU)
 * - the <A HREF="http://www.tbi.univie.ac.at/~ivo/RNA/">Vienna RNA package</A>
 * 
 * \section SecApp Appendix
 * 
 * Additionally, the documentation also features the following parts:
 *  - \ref PageInstallation
 *  - \ref PageTests
 *
 * The following lists and indices are available
 *  - <a class="el" href="annotated.html">List of all classes including brief documentation</a>
 *  - <a class="el" href="namespaces.html">List of all namespaces including brief documentation</a>
 *  - <a class="el" href="files.html">List of all files</a>
 *  - <a class="el" href="hierarchy.html">Class hierarchy</a>
 *  - <a class="el" href="classes.html">Alphabetical class index</a>
 *  - <a class="el" href="namespacemembers.html">Namespace members</a>
 *  - <a class="el" href="functions.html">Class members</a>
 *  - <a class="el" href="globals.html">File members</a>
 */



/**
 * \page PageInstallation Installation
 * 
 * Follow this steps to create a local version of the ELL:
 * 
 * - download and unzip the 
 *   <A HREF="http://www.bioinf.uni-freiburg.de/SW/ELL/"> ELL source package </A> 
 *   distribution
 * 
 * - run "./configure --help" to get an overview of further possible parameters
 * - run "./configure" with your desired parameters to generate the Makefiles
 * - "make" to generate the library
 * 
 * - "make test" to test if everything works fine (if not please report)
 * 
 * - "make install" to install the headers and library to the given directory
 * 
 * - "make doc" to generate the html documentation (if selected during the
 *   configure call)
 * - "make doxygen-doc" to generate the whole documentation
 *
 * See the file INSTALL in the library root path for further informations.
 * 
 */
 
/**
 * \page PageTests Test cases and usage examples
 * 
 * To check if the ELL does function as expected a test case was implemented
 * for each non-abstracted class. 
 * 
 * The source code can be used as an example for the usage of the ELL-classes.
 * 
 * All files can be found in a subfolder named "tests" in the ELL root
 * directory. They are named in the format "testCLASSNAME.cc".
 * 
 */

