
#include "ell/RateMatrix.hh"

#include "biu/assertbiu.hh"

#include <math.h>
#include <limits>

namespace ell {

	////////////////////////////////////////////////////////////////////////////

	RateMatrix::
	RateMatrix(const double defVal)
	  :	biu::MatrixSparseC<double>(defVal)
	{
	}
	
	////////////////////////////////////////////////////////////////////////////
	
	RateMatrix::
	RateMatrix(const size_t r, const size_t c, const double val)
	  :	biu::MatrixSparseC<double>(r,c,val)
	{
	}
	
	////////////////////////////////////////////////////////////////////////////
	
	RateMatrix::
	RateMatrix(const RateMatrix &mat)
	  :	biu::MatrixSparseC<double>(mat)
	{
	}

	////////////////////////////////////////////////////////////////////////////

	// Calculates columnwise an RMSD of the corresponding columns, i.e. an
	// RMSD of the transition rates for each minimum
	// ASSUMPTION: R_orig and R_compare describe the same set of states, such that
	// R_orig.at(to,from) describes the same transition than R_compare.at(to,from). So the
	// dimension of R_orig and R_compare have to be the same!
	// @param R_orig the first rate matrix
	// @param R_compare the second rate matrix
	// @return a column RMSD of the transitions for each state covered by R_orig
	//         and R_compare
	std::vector<double>
	RateMatrix::
	RMSD( const ell::RateMatrix & R_orig, const ell::RateMatrix & R_compare)
	{
		assertbiu(R_orig.numColumns() == R_orig.numRows(), "R_orig no square matrix");
		assertbiu(R_compare.numColumns() == R_compare.numRows(), "R_compare no square matrix");
		assertbiu(R_orig.numColumns() == R_compare.numColumns(), "R_orig and R_compare differ in dimensions");
		
		  // the RMSD vector to fill
		std::vector<double> rmsds(R_orig.numColumns(),0.0);
		
		for (size_t from=0; from < R_orig.numColumns(); from++) {
			  // the square deviation sum
			double diff2sum = 0.0;
			for (size_t to=0; to<R_orig.numRows(); to++) {
				diff2sum +=	 ( R_orig.at(to,from) - R_compare.at(to,from) )
							*( R_orig.at(to,from) - R_compare.at(to,from) );
			}
			  // calculate and store the RMSD of the current "from" column
			rmsds[from] = sqrt(diff2sum/double(R_orig.numRows()));
		}
		
		return rmsds;
	}
	
	
	////////////////////////////////////////////////////////////////////////////

	
	// Calculates columnwise the Kullback-Leibler divergence of the 
	// corresponding columns, i.e. the information divergence of the 
	// transition rates for each minimum which are probability distributions.
	// ASSUMPTION: R_orig and R_compare describe the same set of states, such that
	// R_orig.at(to,from) describes the same transition than R_compare.at(to,from). So the
	// dimension of R_orig and R_compare have to be the same!
	// @param R_orig the original rate matrix
	// @param R_compare the second rate matrix
	// @return the Kullback-Leibler divergence for each column of transitions
	//         for each state covered by R_orig and R_compare
	std::vector<double>
	RateMatrix::
	KLD( const ell::RateMatrix & R_orig, const ell::RateMatrix & R_compare)
	{
		assertbiu(R_orig.numColumns() == R_orig.numRows(), "R_orig no square matrix");
		assertbiu(R_compare.numColumns() == R_compare.numRows(), "R_compare no square matrix");
		assertbiu(R_orig.numColumns() == R_compare.numColumns(), "R_orig and R_compare differ in dimensions");
		
		  // the Kullback-Leibler divergence vector to fill
		std::vector<double> klds(R_orig.numColumns(),0.0);
		
		for (size_t from=0; from < R_orig.numColumns(); from++) {
			  // the sum of divergences
			double sum = 0.0;
			bool allNull = true;
			for (size_t to=0; to<R_orig.numRows(); to++) {
				  // check only for non-zero values in R_compare
				if ( R_orig.at(to,from) > 0.0 && R_compare.at(to,from) > 0.0) {
					  // calculate the KLD contribution for non-negative values only
					sum += R_compare.at(to,from) * log( R_compare.at(to,from) / R_orig.at(to,from) );
					allNull = false;
				}
			}
			if (allNull)
				sum = std::numeric_limits<double>::signaling_NaN();
			  // store the KLD of the current "from" column
			klds[from] = sum;
		}
		
		return klds;
	}


	
	////////////////////////////////////////////////////////////////////////////
	

	void
	RateMatrix::
	normalize( RateMatrix & R )
	{
		RateMatrix::EntryMap col;
		RateMatrix::EntryMap::const_iterator to;
		
		  // normalize rates to get a stochastic matrix
		for (size_t from=0; from<R.numColumns(); from++) {
			col = R.columnValues(from);
			double sum = 0.0;
			  // sum over all non-empty entries;
			for (to=col.begin(); to!=col.end(); to++) {
				sum += to->second;
			}
			  // check if current basin was not seen
			if (sum == 0.0) {
				  // set probability to stay in basin to one
				R.at(from,from) = 1.0;
			} else {
				  // update all non-empty entries;
				for (to=col.begin(); to!=col.end(); to++) {
					R.at(to->first,from) /= sum;
				}
			}
		}

	}

	
	////////////////////////////////////////////////////////////////////////////
	
	
} // namespace ell
