#ifndef DW_RNA_HH_
#define DW_RNA_HH_

#include "ell/DirectWalk.hh"

#include <string>
#include <vector>
	
namespace ell {

	/*!
	   * A DirectWalk instantiation that implements the Morgan-Higgs heuristic
	   * to determine a low energy path between two RNA conformations.
	   * 
	   * Thus, the algorithm is restricted to RNA State implementations.
	   * 
	   * @author: Hannes Kochni, Martin Mann
	   */
	class DW_RNA_MorganHiggs : public DirectWalk {
		
	protected:
		
		  //! number of sampling iterations to perform to lower the already
		  //! found path
		const size_t iterations; 
		
	public:
		
		  //! construction
		  //! @param iterations the number of sampling iterations to perform to
		  //!        lower the already found path
		DW_RNA_MorganHiggs( const size_t iterations );
		
		  //! virtual destructor
		virtual ~DW_RNA_MorganHiggs();
			
		  //! Performs a directed low energy walk between two specified RNA 
		  //! States in the landscape applying the heuristic by Morgan and 
		  //! Higgs.
		  //!
		  //! Note: the begin and end States have to be an RNA State 
		  //! instantiation of S_RNAfe_SingleM.
		  //!
		  //! @param begin the begin of the walk
		  //! @param end the end of the walk
		  //! @param scWalk the StateCollector each step of the walk is added to
		  //! @return the modified scWalk StateCollector
		virtual 
		StateCollector*
		walk(	const State* const begin
				, const State* const end
				, StateCollector* const scWalk
			) const;
		
		  //! Access to the used number of sampling iterations to perform to 
		  //! lower the already found path.
		  //! @return number of sampling iterations
		size_t
		getIterations(void) const;
		
	protected:
		
		static
		double 
		addBond(	std::string& sequence
					, std::vector<size_t>& v_struct
					, size_t start
					, size_t end
					, StateCollector* sc );
		
		static
		double 
		deleteBond(	std::string& sequence
					, std::vector<size_t>& v_struct
					, size_t start
					, StateCollector* sc );

		
	};
	
} // namespace ell



#endif /*DW_RNA_HH_*/
